@@ -19,7 +19,6 @@
 #include "common.h"
 #include <assert.h>
 #include <errno.h>
-#include <inttypes.h>
 #include <signal.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -45,7 +44,7 @@
 #if YASH_ENABLE_LINEEDIT
 # include "xfnmatch.h"
 # include "lineedit/complete.h"
-# if !defined(FG_DONT_SAVE_TERMINAL)
+# ifndef FG_DONT_SAVE_TERMINAL
 #  include "lineedit/terminfo.h"
 # endif
 #endif
@@ -71,7 +70,7 @@ static int print_job_status(
     __attribute__((nonnull));
 static size_t get_jobnumber_from_name(const wchar_t *name)
     __attribute__((nonnull,pure));
-static size_t get_jobnumber_from_pid(pid_t pid)
+static size_t get_jobnumber_from_pid(long pid)
     __attribute__((pure));
 
 static bool jobs_builtin_print_job(size_t jobnumber,
@@ -79,12 +78,13 @@ static bool jobs_builtin_print_job(size_t jobnumber,
 	bool runningonly, bool stoppedonly);
 static int continue_job(size_t jobnumber, job_T *job, bool fg)
     __attribute__((nonnull));
-static bool wait_has_job(bool jobcontrol);
+static bool wait_builtin_has_job(bool jobcontrol);
 
 
 /* The list of jobs.
  * `joblist.contents[ACTIVE_JOBNO]' is a special job that is called "active
- * job": the job that is currently being executed. */
+ * job", the job that is currently being executed.
+ * The list length is always non-zero. */
 static plist_T joblist;
 
 /* number of the current/previous jobs. 0 if none. */
@@ -101,6 +101,7 @@ void init_job(void)
 /* Sets the active job. */
 void set_active_job(job_T *job)
 {
+    assert(ACTIVE_JOBNO < joblist.length);
     assert(joblist.contents[ACTIVE_JOBNO] == NULL);
     joblist.contents[ACTIVE_JOBNO] = job;
 }
@@ -167,35 +168,36 @@ void remove_all_jobs(void)
 /* Frees the specified job. */
 void free_job(job_T *job)
 {
-    if (job) {
+    if (job != NULL) {
 	for (size_t i = 0; i < job->j_pcount; i++)
 	    free(job->j_procs[i].pr_name);
 	free(job);
     }
 }
 
-/* Removes unused elements in `joblist'. */
+/* Shrink the job list, removing unused elements. */
 void trim_joblist(void)
 {
-    size_t tail = joblist.length;
+    if (joblist.maxlength > 20 && joblist.maxlength / 2 > joblist.length) {
+	pl_setmax(&joblist, joblist.length * 2);
+    } else {
+	size_t tail = joblist.length;
 
-    while (tail > 0 && joblist.contents[--tail] == NULL);
-    tail++;
-    assert(tail > 0 && joblist.contents[tail] == NULL);
-    if (joblist.maxlength > 20 && joblist.maxlength / 2 > joblist.length)
-	pl_setmax(&joblist, tail);
-    else
+	while (tail > 1 && joblist.contents[tail - 1] == NULL)
+	    tail--;
+	assert(tail > 0);
 	pl_remove(&joblist, tail, SIZE_MAX);
+    }
 }
 
-/* Negate the `j_pgid' member of all jobs.
- * All the jobs are no longer taken care of by job control. */
+/* Sets the `j_legacy' flags of all jobs.
+ * All the jobs will be no longer job-controlled. */
 void neglect_all_jobs(void)
 {
     for (size_t i = 0; i < joblist.length; i++) {
 	job_T *job = joblist.contents[i];
-	if (job && job->j_pgid >= 0)
-	    job->j_pgid = (job->j_pgid > 0) ? -job->j_pgid : -1;
+	if (job != NULL)
+	    job->j_legacy = true;
     }
     current_jobnumber = previous_jobnumber = 0;
 }
@@ -240,7 +242,7 @@ void set_current_jobnumber(size_t jobnumber)
 	    newcurrent = get_job(jobnumber);
 	    if (newcurrent == NULL
 		    || (stopcount > 0 && newcurrent->j_status != JS_STOPPED))
-		jobnumber = find_next_job(0);
+		jobnumber = find_next_job(ACTIVE_JOBNO);
 	}
     }
 
@@ -303,8 +305,9 @@ void apply_curstop(void)
     if (shopt_curstop) {
 	for (size_t i = 0; i < joblist.length; i++) {
 	    job_T *job = joblist.contents[i];
-	    if (job && job->j_status == JS_STOPPED && job->j_statuschanged)
-		set_current_jobnumber(i);
+	    if (job != NULL)
+		if (job->j_status == JS_STOPPED && job->j_statuschanged)
+		    set_current_jobnumber(i);
 	}
     }
     set_current_jobnumber(current_jobnumber);
@@ -315,7 +318,7 @@ size_t job_count(void)
 {
     size_t count = 0;
     for (size_t i = 0; i < joblist.length; i++)
-	if (joblist.contents[i])
+	if (joblist.contents[i] != NULL)
 	    count++;
     return count;
 }
@@ -326,7 +329,7 @@ size_t stopped_job_count(void)
     size_t count = 0;
     for (size_t i = 0; i < joblist.length; i++) {
 	job_T *job = joblist.contents[i];
-	if (job && job->j_status == JS_STOPPED)
+	if (job != NULL && job->j_status == JS_STOPPED)
 	    count++;
     }
     return count;
@@ -379,12 +382,12 @@ start:
 
     /* determine `jobnumber', `job' and `pr' from `pid' */
     for (jobnumber = 0; jobnumber < joblist.length; jobnumber++)
-	if ((job = joblist.contents[jobnumber]))
+	if ((job = joblist.contents[jobnumber]) != NULL)
 	    for (pnumber = 0; pnumber < job->j_pcount; pnumber++)
 		if ((pr = &job->j_procs[pnumber])->pr_pid == pid)
 		    goto found;
 
-    /* If `pid' is not found in the job list, we simply ignore it. This may
+    /* If `pid' was not found in the job list, we simply ignore it. This may
      * happen on some occasions: e.g. the job has been "disown"ed. */
     goto start;
 
@@ -442,7 +445,7 @@ int wait_for_job(size_t jobnumber, bool return_on_stop,
     int signum = 0;
     job_T *job = joblist.contents[jobnumber];
 
-    if (job->j_pgid >= 0) {
+    if (!job->j_legacy) {
 	bool savenonotify = job->j_nonotify;
 	job->j_nonotify = true;
 	for (;;) {
@@ -469,17 +472,18 @@ int wait_for_job(size_t jobnumber, bool return_on_stop,
  * Traps are not handled in this function.
  * There must be no active job when this function is called.
  * If `return_on_stop' is true and the child is stopped, this function returns
- * a pointer to a pointer to a wide string. The caller may assign a pointer to
+ * a pointer to a pointer to a wide string. The caller must assign a pointer to
  * a newly malloced wide string to the variable the return value points to.
  * This string is used as the name of the new stopped job.
  * If the child exited, this function returns NULL.
- * The exit status is assigned to `laststatus' anyway. */
+ * The exit status is assigned to `laststatus' in any case. */
 wchar_t **wait_for_child(pid_t cpid, pid_t cpgid, bool return_on_stop)
 {
     job_T *job = xmalloc(sizeof *job + sizeof *job->j_procs);
     job->j_pgid = cpgid;
     job->j_status = JS_RUNNING;
     job->j_statuschanged = false;
+    job->j_legacy = false;
     job->j_nonotify = false;
     job->j_pcount = 1;
     job->j_procs[0].pr_pid = cpid;
@@ -507,7 +511,7 @@ wchar_t **wait_for_child(pid_t cpid, pid_t cpgid, bool return_on_stop)
 pid_t get_job_pgid(const wchar_t *jobname)
 {
     size_t jobnumber = get_jobnumber_from_name(
-	    jobname[0] == L'%' ? jobname + 1 : jobname);
+	    (jobname[0] == L'%') ? &jobname[1] : jobname);
     const job_T *job;
 
     if (jobnumber >= joblist.length) {
@@ -515,7 +519,7 @@ pid_t get_job_pgid(const wchar_t *jobname)
 	return -1;
     } else if (jobnumber == 0
 	    || (job = joblist.contents[jobnumber]) == NULL
-	    || job->j_pgid < 0) {
+	    || job->j_legacy) {
 	xerror(0, Ngt("no such job `%ls'"), jobname);
 	return -1;
     } else if (job->j_pgid == 0) {
@@ -604,7 +608,7 @@ int calc_status_of_job(const job_T *job)
 {
     switch (job->j_status) {
     case JS_DONE:
-	if (job->j_procs[job->j_pcount - 1].pr_pid)
+	if (job->j_procs[job->j_pcount - 1].pr_pid != 0)
 	    return calc_status(job->j_procs[job->j_pcount - 1].pr_statuscode);
 	else
 	    return job->j_procs[job->j_pcount - 1].pr_statuscode;
@@ -719,7 +723,9 @@ int print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
     int result = 0;
 
     job_T *job = get_job(jobnumber);
-    if (!job || (changedonly && !job->j_statuschanged) || job->j_nonotify)
+    if (job == NULL
+	    || (changedonly && !job->j_statuschanged)
+	    || job->j_nonotify)
 	return result;
 
     char current;
@@ -791,16 +797,16 @@ void print_job_status_all(void)
 }
 
 /* If the shell is interactive and the specified job has been killed by a
- * signal other than SIGPIPE, prints a notification to stderr.
- * If the signal is SIGINT, only a single newline is printed to stderr and the
- * shell is flagged as interrupted. */
+ * signal other than SIGPIPE, prints a notification to the standard error.
+ * If the signal is SIGINT, only a single newline is printed to the standard
+ * error and the shell is flagged as interrupted. */
 void notify_signaled_job(size_t jobnumber)
 {
     if (!is_interactive_now)
 	return;
 
     job_T *job = get_job(jobnumber);
-    if (!job || job->j_status != JS_DONE)
+    if (job == NULL || job->j_status != JS_DONE)
 	return;
 
     process_T *p = &job->j_procs[job->j_pcount - 1];
@@ -848,7 +854,7 @@ size_t get_jobnumber_from_name(const wchar_t *name)
     if (iswdigit(name[0])) {
 	unsigned long num;
 	if (xwcstoul(name, 10, &num))
-	    return (num <= SIZE_MAX && get_job(num)) ? num : 0;
+	    return (num <= SIZE_MAX && get_job(num) != NULL) ? num : 0;
     }
 
     bool contain;
@@ -861,9 +867,11 @@ size_t get_jobnumber_from_name(const wchar_t *name)
     }
     for (size_t i = 1; i < joblist.length; i++) {
 	job_T *job = joblist.contents[i];
-	if (job) {
+	if (job != NULL) {
 	    wchar_t *jobname = get_job_name(job);
-	    bool match = (contain ? wcsstr : matchwcsprefix)(jobname, name);
+	    bool match = contain
+		? wcsstr(jobname, name) != NULL
+		: matchwcsprefix(jobname, name) != NULL;
 	    if (jobname != job->j_procs[0].pr_name)
 		free(jobname);
 	    if (match) {
@@ -879,14 +887,14 @@ size_t get_jobnumber_from_name(const wchar_t *name)
 
 /* Returns the number of job that contains a process whose process ID is `pid'.
  * If not found, 0 is returned. */
-size_t get_jobnumber_from_pid(pid_t pid)
+size_t get_jobnumber_from_pid(long pid)
 {
     size_t jobnumber;
     if (pid == 0)
 	return 0;
     for (jobnumber = joblist.length; --jobnumber > 0; ) {
 	job_T *job = joblist.contents[jobnumber];
-	if (job) {
+	if (job != NULL) {
 	    for (size_t i = 0; i < job->j_pcount; i++)
 		if (job->j_procs[i].pr_pid == pid)
 		    goto found;
@@ -941,7 +949,7 @@ void generate_job_candidates(const le_compopt_T *compopt)
 #endif /* YASH_ENABLE_LINEEDIT */
 
 
-/********** Builtins **********/
+/********** Built-ins **********/
 
 /* The "jobs" built-in, which accepts the following options:
  *  -l: be verbose
@@ -1002,6 +1010,7 @@ int jobs_builtin(int argc, void **argv)
 			ARGV(xoptind));
 		continue;
 	    }
+
 	    size_t jobnumber = get_jobnumber_from_name(jobspec);
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("job specification `%ls' is ambiguous"),
@@ -1046,7 +1055,7 @@ bool jobs_builtin_print_job(size_t jobnumber,
     if (pgidonly) {
 	if (changedonly && !job->j_statuschanged)
 	    return true;
-	int result = printf("%jd\n", imaxabs(job->j_pgid));
+	int result = printf("%jd\n", (intmax_t) job->j_pgid);
 	err = (result >= 0) ? 0 : errno;
     } else {
 	err = print_job_status(jobnumber, changedonly, verbose, stdout);
@@ -1091,10 +1100,10 @@ const char *jobs_help[] = { Ngt(
 ), NULL };
 #endif
 
-/* The "fg"/"bg" builtin */
+/* The "fg"/"bg" built-in */
 int fg_builtin(int argc, void **argv)
 {
-    bool fg = wcscmp(argv[0], L"fg") == 0;
+    bool fg = (wcscmp(argv[0], L"fg") == 0);
 
     const struct xgetopt_T *opt;
     xoptind = 0;
@@ -1133,13 +1142,14 @@ int fg_builtin(int argc, void **argv)
 			ARGV(xoptind));
 		continue;
 	    }
+
 	    size_t jobnumber = get_jobnumber_from_name(jobspec);
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("job specification `%ls' is ambiguous"),
 			ARGV(xoptind));
 	    } else if (jobnumber == 0
 		    || (job = joblist.contents[jobnumber]) == NULL
-		    || job->j_pgid < 0) {
+		    || job->j_legacy) {
 		xerror(0, Ngt("no such job `%ls'"), ARGV(xoptind));
 	    } else if (job->j_pgid == 0) {
 		xerror(0, Ngt("`%ls' is not a job-controlled job"),
@@ -1150,7 +1160,7 @@ int fg_builtin(int argc, void **argv)
 	} while (++xoptind < argc);
     } else {
 	if (current_jobnumber == 0 ||
-		(job = joblist.contents[current_jobnumber])->j_pgid <= 0) {
+		(job = joblist.contents[current_jobnumber])->j_legacy) {
 	    xerror(0, Ngt("there is no current job"));
 	} else {
 	    status = continue_job(current_jobnumber, job, fg);
@@ -1169,6 +1179,7 @@ int fg_builtin(int argc, void **argv)
 int continue_job(size_t jobnumber, job_T *job, bool fg)
 {
     assert(job->j_pgid > 0);
+    assert(!job->j_legacy);
 
     wchar_t *name = get_job_name(job);
     if (fg && posixly_correct)
@@ -1258,7 +1269,7 @@ const char *bg_help[] = { Ngt(
 
 #endif /* YASH_ENABLE_HELP */
 
-/* The "wait" builtin */
+/* The "wait" built-in */
 int wait_builtin(int argc, void **argv)
 {
     bool jobcontrol = doing_job_control_now;
@@ -1285,7 +1296,7 @@ int wait_builtin(int argc, void **argv)
 	    const wchar_t *jobspec = ARGV(xoptind);
 	    size_t jobnumber;
 	    if (jobspec[0] == L'%') {
-		jobnumber = get_jobnumber_from_name(jobspec + 1);
+		jobnumber = get_jobnumber_from_name(&jobspec[1]);
 	    } else {
 		long pid;
 		if (!xwcstol(jobspec, 10, &pid) || pid < 0) {
@@ -1293,15 +1304,14 @@ int wait_builtin(int argc, void **argv)
 			    jobspec);
 		    continue;
 		}
-		jobnumber = get_jobnumber_from_pid((pid_t) pid);
-		// XXX This cast might not be safe
+		jobnumber = get_jobnumber_from_pid(pid);
 	    }
 	    if (jobnumber >= joblist.length) {
 		xerror(0, Ngt("job specification `%ls' is ambiguous"),
 			ARGV(xoptind));
 	    } else if (jobnumber == 0
 		    || (job = joblist.contents[jobnumber]) == NULL
-		    || job->j_pgid < 0) {
+		    || job->j_legacy) {
 		status = Exit_NOTFOUND;
 	    } else {
 		status = wait_for_job(jobnumber, jobcontrol, jobcontrol, true);
@@ -1322,7 +1332,7 @@ int wait_builtin(int argc, void **argv)
 	} while (++xoptind < argc);
     } else {
 	/* wait for all jobs */
-	while (wait_has_job(jobcontrol)) {
+	while (wait_builtin_has_job(jobcontrol)) {
 	    status = wait_for_sigchld(jobcontrol, true);
 	    if (status) {
 		assert(TERMSIGOFFSET >= 128);
@@ -1340,7 +1350,7 @@ int wait_builtin(int argc, void **argv)
 }
 
 /* Checks if the shell has any job to wait for. */
-bool wait_has_job(bool jobcontrol)
+bool wait_builtin_has_job(bool jobcontrol)
 {
     /* print/remove already-finished jobs */
     if (jobcontrol && is_interactive_now && !posixly_correct) {
@@ -1349,7 +1359,7 @@ bool wait_has_job(bool jobcontrol)
     } else {
 	for (size_t i = 1; i < joblist.length; i++) {
 	    job_T *job = joblist.contents[i];
-	    if (job != NULL && (job->j_pgid < 0 || job->j_status == JS_DONE))
+	    if (job != NULL && (job->j_legacy || job->j_status == JS_DONE))
 		remove_job(i);
 	}
     }
@@ -1376,7 +1386,7 @@ const char *wait_help[] = { Ngt(
 ), NULL };
 #endif
 
-/* The "disown" builtin, which accepts the following option:
+/* The "disown" built-in, which accepts the following option:
  *  -a: disown all jobs */
 int disown_builtin(int argc, void **argv)
 {
@@ -1386,7 +1396,9 @@ int disown_builtin(int argc, void **argv)
     xoptind = 0;
     while ((opt = xgetopt(argv, all_option, 0)) != NULL) {
 	switch (opt->shortopt) {
-	    case L'a':  all = true;  break;
+	    case L'a':
+		all = true;
+		break;
 #if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
