@@ -244,6 +244,7 @@ bool is_update(const char *path)
 {
     struct stat st;
     if (stat(path, &st) < 0) {
+	st.st_size = 0;
 	st.st_mtime = 0;
 #if HAVE_ST_MTIM
 	st.st_mtim.tv_nsec = 0;
@@ -273,7 +274,7 @@ bool is_update(const char *path)
 #endif
 	    );
     } else {
-	mf = xmalloc(sizeof *mf + strlen(path) + 1);
+	mf = xmallocs(sizeof *mf, strlen(path) + 1, sizeof *mf->mf_filename);
 	strcpy(mf->mf_filename, path);
 	ht_set(&mailfiles, mf->mf_filename, mf);
 	result = false;
