@@ -109,30 +109,32 @@ bool write_all(int fd, const void *data, size_t size)
     return true;
 }
 
-/********** Shell FD **********/
+
+/********** Shell FDs **********/
 
 static void reset_shellfdmin(void);
 
 
-/* true iff stdin is redirected */
+/* True iff the standard input is redirected */
 static bool is_stdin_redirected = false;
 
-/* set of file descriptors used by the shell.
+/* Set of file descriptors used by the shell.
  * These file descriptors cannot be used by the user. */
 static fd_set shellfds;
-/* the minimum file descriptor that can be used for shell FD. */
+/* The minimum file descriptor that can be used for shell FD. */
 static int shellfdmin;
-/* the maximum file descriptor in `shellfds'.
- * `shellfdmax' is -1 if `shellfds' is empty. */
-static int shellfdmax;
+/* The maximum file descriptor in `shellfds'.
+ * `shellfdmax' is -1 when `shellfds' is empty. */
+static int shellfdmax = -1;
 
 #ifndef SHELLFDMINMAX
 #define SHELLFDMINMAX 100  /* maximum for `shellfdmin' */
-#elif SHELLFDMINMAX < 10
+#endif
+#if SHELLFDMINMAX < 10
 #error SHELLFDMINMAX too little
 #endif
 
-/* file descriptor associated with the controlling terminal */
+/* File descriptor associated with the controlling terminal */
 int ttyfd = -1;
 
 
@@ -147,7 +149,7 @@ void init_shellfds(void)
 
     FD_ZERO(&shellfds);
     reset_shellfdmin();
-    shellfdmax = -1;
+    assert(shellfdmax == -1);  // shellfdmax = -1;
 }
 
 /* Recomputes `shellfdmin'. */
@@ -155,12 +157,14 @@ void reset_shellfdmin(void)
 {
     errno = 0;
     shellfdmin = sysconf(_SC_OPEN_MAX);
-    if (shellfdmin == -1) {
-	if (errno)
+    if (shellfdmin < 0) {
+	if (errno != 0)
 	    shellfdmin = 10;
 	else
 	    shellfdmin = SHELLFDMINMAX;
     } else {
+	if (shellfdmin > FD_SETSIZE)
+	    shellfdmin = FD_SETSIZE;
 	shellfdmin /= 2;
 	if (shellfdmin > SHELLFDMINMAX)
 	    shellfdmin = SHELLFDMINMAX;
@@ -186,9 +190,9 @@ void remove_shellfd(int fd)
     if (0 <= fd && fd < FD_SETSIZE)
 	FD_CLR(fd, &shellfds);
     if (fd == shellfdmax) {
-	shellfdmax = fd - 1;
-	while (shellfdmax >= 0 && !FD_ISSET(shellfdmax, &shellfds))
+	do
 	    shellfdmax--;
+	while (shellfdmax >= 0 && !FD_ISSET(shellfdmax, &shellfds));
     }
 }
 /* The argument to `FD_CLR' must be a valid (open) file descriptor. This is why
@@ -245,6 +249,23 @@ finish:
     return newfd;
 }
 
+/* Moves the specified file descriptor (FD) to a shell FD.
+ * The original FD is closed (whether successful or not).
+ * If `fd' is negative, this function simply returns `fd' (without changing
+ * `errno'). If `fd' is non-negative and the FD cannot be copied, `errno' is set
+ * to indicate the error. */
+int move_to_shellfd(int fd)
+{
+    if (fd < 0)
+	return fd;
+
+    int newfd = copy_as_shellfd(fd);
+    int saveerrno = errno;
+    xclose(fd);
+    errno = saveerrno;
+    return newfd;
+}
+
 /* Opens `ttyfd'.
  * On failure, an error message is printed and `do_job_control' is set to false.
  */
@@ -297,10 +318,10 @@ static int open_process_redirection(const embedcmd_T *command, redirtype_T type)
  * Returns true iff successful. */
 bool open_redirections(const redir_T *r, savefd_T **save)
 {
-    if (save)
+    if (save != NULL)
 	*save = NULL;
 
-    while (r) {
+    while (r != NULL) {
 	if (r->rd_fd < 0 || is_shellfd(r->rd_fd)) {
 	    xerror(0, Ngt("redirection: file descriptor %d is unavailable"),
 		    r->rd_fd);
@@ -314,7 +335,7 @@ bool open_redirections(const redir_T *r, savefd_T **save)
 	    case RT_INOUT:  case RT_DUPIN:   case RT_DUPOUT:   case RT_PIPE:
 	    case RT_HERESTR:
 		filename = expand_redir_filename(r->rd_filename);
-		if (!filename)
+		if (filename == NULL)
 		    return false;
 		break;
 	    default:
@@ -425,7 +446,7 @@ char *expand_redir_filename(const struct wordunit_T *filename)
 	if (result == NULL)
 	    return NULL;
 	char *mbsresult = realloc_wcstombs(unescapefree(result));
-	if (!mbsresult)
+	if (mbsresult == NULL)
 	    xerror(EILSEQ, Ngt("redirection"));
 	return mbsresult;
     }
@@ -435,7 +456,7 @@ char *expand_redir_filename(const struct wordunit_T *filename)
 void save_fd(int fd, savefd_T **save)
 {
     assert(fd >= 0);
-    if (!save)
+    if (save == NULL)
 	return;
 
     int copyfd = copy_as_shellfd(fd);
@@ -462,19 +483,17 @@ void save_fd(int fd, savefd_T **save)
  * -1 is returned. */
 int open_file(const char *path, int oflag)
 {
-    int fd;
-
-    fd = open(path, oflag,
+    int fd = open(path, oflag,
 	    S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 #if YASH_ENABLE_SOCKET
     if (fd < 0) {
 	const char *hostandport = matchstrprefix(path, "/dev/tcp/");
-	if (hostandport)
+	if (hostandport != NULL)
 	    fd = open_socket(hostandport, SOCK_STREAM);
     }
     if (fd < 0) {
 	const char *hostandport = matchstrprefix(path, "/dev/udp/");
-	if (hostandport)
+	if (hostandport != NULL)
 	    fd = open_socket(hostandport, SOCK_DGRAM);
     }
 #endif /* YASH_ENABLE_SOCKET */
@@ -503,14 +522,14 @@ int open_socket(const char *hostandport, int socktype)
 	const wchar_t *wport;
 
 	whostandport = malloc_mbstowcs(hostandport);
-	if (!whostandport) {
+	if (whostandport == NULL) {
 	    errno = saveerrno;
 	    return -1;
 	}
 	wport = wcschr(whostandport, L'/');
-	if (wport) {
+	if (wport != NULL) {
 	    hostname = malloc_wcsntombs(whostandport, wport - whostandport);
-	    port = malloc_wcstombs(wport + 1); // TODO return value null check
+	    port = malloc_wcstombs(wport + 1); // XXX error ignored
 	} else {
 	    hostname = xstrdup(hostandport);
 	    port = NULL;
@@ -572,17 +591,17 @@ int parse_and_check_dup(char *const num, redirtype_T type)
 
     if (!xisxdigit(num[0]))
 	errno = EINVAL;
-    else if (xstrtoi(num, 10, &fd) && fd < 0)
-	errno = ERANGE;
-    if (errno) {
+    else if (xstrtoi(num, 10, &fd))
+	if (fd < 0)
+	    errno = ERANGE;
+    if (errno != 0) {
 	xerror(errno, Ngt("redirection: %s"), num);
 	fd = -2;
 	goto end;
     }
 
     if (is_shellfd(fd)) {
-	xerror(0, Ngt("redirection: file descriptor %d is unavailable"),
-		fd);
+	xerror(0, Ngt("redirection: file descriptor %d is unavailable"), fd);
 	fd = -2;
 	goto end;
     }
@@ -645,7 +664,7 @@ int parse_and_exec_pipe(int outputfd, char *num, savefd_T **save)
 	if (xstrtoi(num, 10, &inputfd) && inputfd < 0)
 	    errno = ERANGE;
     }
-    if (errno) {
+    if (errno != 0) {
 	xerror(errno, Ngt("redirection: %s"), num);
 	fd = -1;
     } else if (outputfd == inputfd) {
@@ -664,32 +683,34 @@ int parse_and_exec_pipe(int outputfd, char *num, savefd_T **save)
 	    goto error;
 
 	/* move the output side from what is to be the input side. */
-	if (pipefd[PIDX_OUT] == inputfd) {
-	    int newfd = dup(pipefd[PIDX_OUT]);
+	if (pipefd[PIPE_OUT] == inputfd) {
+	    int newfd = dup(pipefd[PIPE_OUT]);
 	    if (newfd < 0)
 		goto error2;
-	    xclose(pipefd[PIDX_OUT]);
-	    pipefd[PIDX_OUT] = newfd;
+	    xclose(pipefd[PIPE_OUT]);
+	    pipefd[PIPE_OUT] = newfd;
 	}
 
 	/* move the input side to where it should be. */
-	if (pipefd[PIDX_IN] != inputfd) {
-	    if (xdup2(pipefd[PIDX_IN], inputfd) < 0)
+	if (pipefd[PIPE_IN] != inputfd) {
+	    if (xdup2(pipefd[PIPE_IN], inputfd) < 0)
 		goto error2;
-	    xclose(pipefd[PIDX_IN]);
-	    // pipefd[PIDX_IN] = inputfd;
+	    xclose(pipefd[PIPE_IN]);
+	    // pipefd[PIPE_IN] = inputfd;
 	}
 
 	/* The output side is not moved in this function. */
-	fd = pipefd[PIDX_OUT];
+	fd = pipefd[PIPE_OUT];
     }
 end:
     free(num);
     return fd;
 
-error2:
-    xclose(pipefd[PIDX_IN]);
-    xclose(pipefd[PIDX_OUT]);
+error2:;
+    int saveerrno = errno;
+    xclose(pipefd[PIPE_IN]);
+    xclose(pipefd[PIPE_OUT]);
+    errno = saveerrno;
 error:
     xerror(errno, Ngt("redirection: %d>>|%d"), outputfd, inputfd);
     fd = -1;
@@ -703,11 +724,11 @@ error:
 int open_heredocument(const wordunit_T *contents)
 {
     wchar_t *wcontents = expand_string(contents, true);
-    if (!wcontents)
+    if (wcontents == NULL)
 	return -1;
 
     char *mcontents = realloc_wcstombs(wcontents);
-    if (!mcontents) {
+    if (mcontents == NULL) {
 	xerror(EILSEQ, Ngt("cannot write the here-document contents "
 		    "to the temporary file"));
 	return -1;
@@ -747,12 +768,12 @@ int open_herestring(char *s, bool appendnewline)
 	if (pipe(pipefd) >= 0) {
 	    /* It is guaranteed that all the contents is written to the pipe
 	     * at once, so we don't have to use `write_all' here. */
-	    if (write(pipefd[PIDX_OUT], s, len) < 0)
+	    if (write(pipefd[PIPE_OUT], s, len) < 0)
 		xerror(errno, Ngt("cannot write the here-document contents "
 			    "to the temporary file"));
-	    xclose(pipefd[PIDX_OUT]);
+	    xclose(pipefd[PIPE_OUT]);
 	    free(s);
-	    return pipefd[PIDX_IN];
+	    return pipefd[PIPE_IN];
 	}
     }
 #endif /* defined(PIPE_BUF) */
@@ -795,33 +816,33 @@ int open_process_redirection(const embedcmd_T *command, redirtype_T type)
     cpid = fork_and_reset(-1, false, 0);
     if (cpid < 0) {
 	/* fork failure */
-	xclose(pipefd[PIDX_IN]);
-	xclose(pipefd[PIDX_OUT]);
+	xclose(pipefd[PIPE_IN]);
+	xclose(pipefd[PIPE_OUT]);
 	return -1;
     } else if (cpid) {
 	/* parent process */
 	if (type == RT_PROCIN) {
-	    xclose(pipefd[PIDX_OUT]);
-	    return pipefd[PIDX_IN];
+	    xclose(pipefd[PIPE_OUT]);
+	    return pipefd[PIPE_IN];
 	} else {
-	    xclose(pipefd[PIDX_IN]);
-	    return pipefd[PIDX_OUT];
+	    xclose(pipefd[PIPE_IN]);
+	    return pipefd[PIPE_OUT];
 	}
     } else {
 	/* child process */
 	if (type == RT_PROCIN) {
-	    xclose(pipefd[PIDX_IN]);
-	    if (pipefd[PIDX_OUT] != STDOUT_FILENO) {
-		if (xdup2(pipefd[PIDX_OUT], STDOUT_FILENO) < 0)
+	    xclose(pipefd[PIPE_IN]);
+	    if (pipefd[PIPE_OUT] != STDOUT_FILENO) {
+		if (xdup2(pipefd[PIPE_OUT], STDOUT_FILENO) < 0)
 		    exit(Exit_NOEXEC);
-		xclose(pipefd[PIDX_OUT]);
+		xclose(pipefd[PIPE_OUT]);
 	    }
 	} else {
-	    xclose(pipefd[PIDX_OUT]);
-	    if (pipefd[PIDX_IN] != STDIN_FILENO) {
-		if (xdup2(pipefd[PIDX_IN], STDIN_FILENO) < 0)
+	    xclose(pipefd[PIPE_OUT]);
+	    if (pipefd[PIPE_IN] != STDIN_FILENO) {
+		if (xdup2(pipefd[PIPE_IN], STDIN_FILENO) < 0)
 		    exit(Exit_NOEXEC);
-		xclose(pipefd[PIDX_IN]);
+		xclose(pipefd[PIPE_IN]);
 	    }
 	}
 	if (command->is_preparsed)
@@ -835,7 +856,7 @@ int open_process_redirection(const embedcmd_T *command, redirtype_T type)
 /* Restores the saved file descriptor and frees `save'. */
 void undo_redirections(savefd_T *save)
 {
-    while (save) {
+    while (save != NULL) {
 	if (save->sf_copyfd >= 0) {
 	    remove_shellfd(save->sf_copyfd);
 	    xdup2(save->sf_copyfd, save->sf_origfd);
@@ -855,7 +876,7 @@ void undo_redirections(savefd_T *save)
  * The copied FDs are closed. */
 void clear_savefd(savefd_T *save)
 {
-    while (save) {
+    while (save != NULL) {
 	if (save->sf_copyfd >= 0) {
 	    remove_shellfd(save->sf_copyfd);
 	    xclose(save->sf_copyfd);
@@ -879,11 +900,14 @@ void maybe_redirect_stdin_to_devnull(void)
 	    || is_stdin_redirected)
 	return;
 
-    xclose(STDIN_FILENO);
+    if (xclose(STDIN_FILENO) < 0)
+	return;
+
     fd = open("/dev/null", O_RDONLY);
-    if (fd >= 0 && fd != STDIN_FILENO) {  // shouldn't happen, but just in case
-	xdup2(fd, STDIN_FILENO);
-	xclose(fd);
+    if (fd < 0) {
+	//xerror(errno, Ngt("cannot redirect the standard input to /dev/null"));
+    } else {
+	assert(fd == STDIN_FILENO);
     }
     is_stdin_redirected = true;
 }
