@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* redir.h: manages file descriptors and provides functions for redirections */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 #ifndef YASH_REDIR_H
 #define YASH_REDIR_H
 
-#include <stdio.h>
-
 
 extern int xclose(int fd);
 extern int xdup2(int oldfd, int newfd);
@@ -36,35 +34,20 @@ extern _Bool is_shellfd(int fd)
     __attribute__((pure));
 extern void clear_shellfds(_Bool leavefds);
 extern int copy_as_shellfd(int fd);
-static inline int move_to_shellfd(int fd);
+extern int move_to_shellfd(int fd);
 extern void open_ttyfd(void);
 extern int get_ttyfd(void) __attribute__((pure));
 
 typedef struct savefd_T savefd_T;
 struct redir_T;
 
-extern _Bool open_redirections(const struct redir_T *r, struct savefd_T **save);
+extern _Bool open_redirections(const struct redir_T *r, savefd_T **save);
 extern void undo_redirections(savefd_T *save);
 extern void clear_savefd(savefd_T *save);
 extern void maybe_redirect_stdin_to_devnull(void);
 
-#define PIDX_IN  0   /* index of the reading end of a pipe */
-#define PIDX_OUT 1   /* index of the writing end of a pipe */
-
-
-/* Moves the specified file descriptor (FD) to a shell FD.
- * The original FD is closed (whether successful or not).
- * If `fd' is negative, this function simply returns `fd' (without changing
- * `errno'). */
-int move_to_shellfd(int fd)
-{
-    if (fd < 0)
-	return fd;
-
-    int newfd = copy_as_shellfd(fd);
-    xclose(fd);
-    return newfd;
-}
+#define PIPE_IN  0   /* index of the reading end of a pipe */
+#define PIPE_OUT 1   /* index of the writing end of a pipe */
 
 
 #endif /* YASH_REDIR_H */
