@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* variable.h: deals with shell variables and parameters */
-/* (C) 2007-2009 magicant */
+/* (C) 2007-2011 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,6 +80,7 @@ typedef enum path_T {
 
 extern unsigned long current_lineno;
 
+extern void init_environment(void);
 extern void init_variables(void);
 
 typedef enum scope_T {
@@ -129,8 +130,8 @@ extern _Bool parse_dirstack_index(
 	const wchar_t **restrict entryp, _Bool printerror)
     __attribute__((nonnull));
 
-extern const struct xoption *const cd_options;
-extern const struct xoption *const pwd_options;
+extern const struct xgetopt_T *const cd_options;
+extern const struct xgetopt_T *const pwd_options;
 
 extern int typeset_builtin(int argc, void **argv)
     __attribute__((nonnull));
@@ -150,8 +151,8 @@ extern int popd_builtin(int argc, void **argv)
     __attribute__((nonnull));
 extern int dirs_builtin(int argc, void **argv)
     __attribute__((nonnull));
-extern const char typeset_help[], array_help[], unset_help[], shift_help[],
-       getopts_help[], read_help[], pushd_help[], popd_help[], dirs_help[];
+extern const char *typeset_help[], *array_help[], *unset_help[], *shift_help[],
+       *getopts_help[], *read_help[], *pushd_help[], *popd_help[], *dirs_help[];
 
 
 #endif /* YASH_VARIABLE_H */
